#define F_CPU 7372800UL			//dle krystalu

#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <math.h>
#include <avr/sleep.h>

#include "ledmatrix7219d88/ledmatrix7219d88.h"			//knihovny pro adi MAX7219
#include "ledmatrix7219d88/ledmatrix7219d88.c"
#include "mpu6050/mpu6050.h"							// knihovny pro MPU-6050
#include "mpu6050/mpu6050.c"
#include "mylib.c"										//vlastn pomocn knihovna

#define sens_shake 2500			//nastaveni citlivosti pro detekci zatreseni
#define sens_sphere 100			//nastaveni citlivost pro program kulicka
#define sens_use 500			//citlivost, zda je pouzivan

volatile unsigned char prerus_napajeni=0;
volatile unsigned char prerus_gyro=0;
volatile unsigned char prerus_zapinani=0;
volatile unsigned char prerus_vypinani=0;

ISR(TIMER0_COMP_vect)			//vektor preruseni
{
	prerus_gyro=1;
	prerus_napajeni++;
	prerus_vypinani++;
	prerus_zapinani++;
}

int main(void)
{
	ledmatrix7219d88_init();			//iniciace LED matice
	_delay_ms(50);
	ledmatrix7219d88_setintensity(0,8);	//nastaveni intenzity LED
	_delay_ms(50);
	mpu6050_init();						//iniciace MPU-6050
	_delay_ms(50);
	nastaveni();						//nastaveni timeru a AD prevodniku
	_delay_ms(50);
	
	int16_t ax = 0;						//iniciace promennych
	int16_t ay = 0;
	int16_t az = 0;
	int16_t gx = 0;
	int16_t gy = 0;
	int16_t gz = 0;
	double axg = 0;
	double ayg = 0;
	double azg = 0;
	double gxds = 0;
	double gyds = 0;
	double gzds = 0;
	signed char position[] = {4,4};
	signed char horizontal = 0;
	signed char vertical = 0;
	char led = 0;
	int tmp = 0;
	int time = 1;
	char program = 1;
	double natoceni = 0;
	char vypinani = 0;
	char zapinani = 0;
	char zapnuto = 1;
	char napeti_ok = 1;
	char prvni=0;

	UAI();
	switch_program(3);
	writenumber(5);

	while(1)
	{
		if(prerus_napajeni>200)			//kazde 4s kontrola napeti
		{
			prerus_napajeni=0;
			if(!kontrola_napeti())		//nizke napeti
			{
				if(zapnuto==1)
					lowbattery();
				zapnuto=0;
				napeti_ok=0;
			}
			else
				napeti_ok=1;
		}
		
		if(zapnuto==0)				//vypnuto
		{
			if(prerus_zapinani>25)	//0.5s
			{
				prerus_zapinani=0;
				if(napeti_ok==1)
				{
					probud_mems();
					mpu6050_getRawData(&ax, &ay, &az, &gx, &gy, &gz);		//ziskani surovych dat
					_delay_ms(10);
					if(az < -1900 && az > -2200)
					{
						zapinani++;
						if(zapinani>2)		//1s
						{
							zapnuto=1;
							probud_led();
							UAI();
							switch_program(program-1);
							zapinani=0;
							prvni=1;
						}
						else 
						{
							uspi_mems();
							uspi_cpu();
						}
					}
					else
					{
						uspi_mems();
						zapinani=0;	
						uspi_cpu();
					}
				}
				else
					uspi_cpu();		
			}
			else
				uspi_cpu();			
		}	
			
		if(zapnuto==1)				//zapnuto
		{	
			mpu6050_getRawData(&ax, &ay, &az, &gx, &gy, &gz);		//ziskani surovych dat
			
			if(az>2*sens_shake||az<-2*sens_shake)	//zmena programu
			{
				program = switch_program(program);
				prvni=1;
			}
			
			if(prvni==1)
			{
				prvni=0;
				switch(program)
				{
					case 1:
						writenumber(time);
						break;
					case 2:
						position[0]=4;
						position[1]=4;
						tmp=0;
						break;
					case 3:
						led=0;
						natoceni=0;
						break;
				}
			}
			
			switch(program)
			{
				case 1: //kostka
					time++;					//cyklicke pocitani do 6 = "nahodne" cislo 1-6
					if(time > 6)
						time = 1;
					if(ax>sens_shake||ax<-sens_shake||ay>sens_shake||ax<-sens_shake)	//kontrola hodu
					{
						writenumber(time);	//vykresleni kostky
					}
					break;
				
				case 2: //kulicka
					if(ax > sens_sphere)					//posun v ose x
					{
						horizontal-=(ax/sens_sphere);
						if(horizontal < -2)
						{
							position[0]--;
							horizontal = 0;
						}
					}
					
					if(ax < -sens_sphere)
					{
						horizontal-=(ax/sens_sphere);
						if(horizontal > 2)
						{
							position[0]++;
							horizontal = 0;
						}
					}
					
					if(position[0] > 7)					//zarazka na hrane led matice v ose x
						position[0]=7;
					if(position[0] < 0)
						position[0]=0;
					
					if(ay > sens_sphere)					//posun v ose y
					{
						vertical+=(ay/sens_sphere);
						if(vertical > 2)
						{
							position[1]++;
							vertical = 0;
						}
					}
					
					if(ay < -sens_sphere)
					{
						vertical+=(ay/sens_sphere);
						if(vertical < -2)
						{
							position[1]--;
							vertical = 0;
						}
					}
					
					if(position[1] > 7)					//zarazka na hrane led matice v ose y
						position[1]=7;
					if(position[1] < 0)
						position[1]=0;
					
					led = position[1]+position[0]*8;	//prepocet na cislo led
					
					if(tmp != led)						//vykresleni "kulicky"
					{
						ledmatrix7219d88_setledon(0,led);
						ledmatrix7219d88_setledoff(0,tmp);
						tmp = led;
					}
					_delay_ms(30);
					break;
					
				case 3: //gyro
					mpu6050_getConvData(&axg, &ayg, &azg, &gxds, &gyds, &gzds);		//ziskani konv. dat
					if(prerus_gyro==1)		//reakce na vektor preruseni
					{
						gzds=(gzds*0.02)-0.03;	//prepocet st/0.02s + korekce
						natoceni+=gzds;			//suma -> relativni natoceni ve stupnich
						prerus_gyro=0;
					}
					if(natoceni>0)				//prevedeni na uhel 0-359 st.
						natoceni-=360*(int)(natoceni/360);
					else if(natoceni<0)
						natoceni+=360*(1+(int)(natoceni/360));
					led = (int)(natoceni/12.86);	//360/28 obvodovych led
					
					if(tmp != led)
					{
						strelka(led);		//vykresleni strelky
						tmp = led;
					}
					break;
			}
			
			if(prerus_vypinani>12)	//kontrola podminky vypnuti kazdych 0.24s
			{
				prerus_vypinani=0;
				if(az > 1900)
				{
					vypinani++;
					if(vypinani>21)		//21->5.04s
					{
						ind_uspani();
						zapnuto=0;
						vypinani=0;
						uspi_led();
						uspi_mems();
					}
				}
				else
					vypinani=0;
			}
		}
	}
}