#define F_CPU 7372800UL
#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <math.h>
#include <avr/sleep.h>

void nastaveni(void);
void UAI(void);							//logo ustavu UAI
int switch_program(int program);		//prepinani programu
unsigned int kontrola_napeti(void);		//kontrola napeti na baterii
unsigned int read_adc(void);			//cteni napeti pomoci ad prevodniku
void clearmatrix(void);					//zhasnuti matice
void writenumber(int number);			//6 moznosti pro hraci kostku
void writedot(int corner);				//pomocna funkce
void lowbattery(void);					//vystraha slabe baterie
void strelka(int led);					//reprezentace strelky kompasu
void uspi_led(void);					//uspani radice
void uspi_mems(void);					//uspani senzoru
void uspi_cpu(void);					//uspani atmegy
void probud_led(void);					//probuzeni radice
void probud_mems(void);					//probuzeni senzoru
void ind_uspani(void);					//indikace uspani


void nastaveni(void)
{
	// Timer0 iniciace
	TCCR0=(1<<WGM01) | (1<<CS02) | (1<<CS00);
	TCNT0=0x00;
	OCR0=144;//0,02s
	TIMSK=(1<<OCIE0);
	
	//ADC
	ADMUX=0xC0;				//kanl PA0, vnitn reference
	ACSR=0x80;				//odpojen napjen kompartoru
	SFIOR=0x00;				//vypnut multiplexeru kompartoru 				
	ADCSRA=0x87; 			//nastaveni frekvence AD prevodniku, povoleni ADC pevodnku
	
	sei();					//globalni povoleni preruseni
}
	
void UAI(void)
{
	int text [8][24] = {	{0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,0,1,0,0,0,1,1,1,0},
							{0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,1,0,1,0,0,0,1,0,0},
							{0,0,0,0,0,0,0,1,0,0,0,0,1,0,1,0,0,0,1,0,0,1,0,0},
							{0,0,0,0,0,0,0,1,0,0,0,0,1,0,1,1,1,1,1,0,0,1,0,0},
							{0,0,0,0,0,0,0,1,0,0,0,0,1,0,1,0,0,0,1,0,0,1,0,0},
							{0,0,0,0,0,0,0,1,0,0,0,0,1,0,1,0,0,0,1,0,0,1,0,0},
							{0,0,0,0,0,0,0,1,0,0,0,0,1,0,1,0,0,0,1,0,0,1,0,0},
							{0,0,0,0,0,0,0,0,1,1,1,1,0,0,1,0,0,0,1,0,1,1,1,0}
	};
	for(int i=0;i<24;i++)
	{
		for(int j=0;j<8;j++)
		{
			for(int k=0;k<8;k++)
				{
					if(text[j][k]==1)
						ledmatrix7219d88_setledon(0,k+8*j);
				}
		}
		_delay_ms(100);
		for(int j=0;j<8;j++)
		{
			for(int k=0;k<24;k++)
			{				
				text[j][k]=text[j][k+1];
			}
		}
		clearmatrix();
	}
	
}
int switch_program (int program)
{
	clearmatrix();
	program++;
	if(program>3)
		program = 1;
	char kostka [38]={2,3,4,5,6,7,9,14,15,16,17,18,19,20,21,23,24,29,31,32
		,34,35,37,39,40,42,43,45,47,48,53,54,56,57,58,59,60,61};	
	char sipka [28] = {3,4,10,11,12,13,17,18,19,20,21,22,24,25,27,28,30,31,32,35,36,39,43,44,51,52,59,60};
	switch(program)
	{
		case 1:			
			for(int i=0;i<5;i++)
			{
				for(int j=0;j<38;j++)
					ledmatrix7219d88_setledon(0,kostka[j]);
				_delay_ms(100);
				clearmatrix();
				_delay_ms(100);
			}
			break;
		case 2:
			for(int i=0;i<8;i++)
			{
				writedot(i*8+i);
				_delay_ms(60);
				clearmatrix();
			}
			break;
		case 3:
			for(int i=0;i<5;i++)
			{
				for(int j=0;j<28;j++)
				ledmatrix7219d88_setledon(0,sipka[j]);
				_delay_ms(100);
				clearmatrix();
				_delay_ms(100);
			}
			break;	
	}
	
	return program;
}

unsigned int kontrola_napeti(void)
{
	unsigned int hodnota=0;
	for(int i=0;i<10;i++)
	{
		hodnota += read_adc();
	}
	hodnota/=10;
	if(hodnota<700)			//3,5V odpovida hodnote cca 700
		return 0;
	else
		return 1;
}

unsigned int read_adc(void)
  {			 
  ADCSRA|=0x40; 					/*Bit 6 - spout celou konverzi. Pi ukonen pevodu je tento bit hardwarov nulovn */
  while ((ADCSRA & 0x10)==0); 		/*opakuj dokud 4.bit roven 0 (AND) - po dokonen konverze 
  									se hardvarov nastav 4.bit na 1*/
  ADCSRA|=0x10;						//vynuluj 4. bit (OR)
  return ADCW;  					/*ADCW jsou spojene registry ADCL a ADCH 
  									ve kterch je vsledek pevodu*/
  }
  
void clearmatrix(void)
{
	for(int i=0; i<64; i++)
		ledmatrix7219d88_setledoff(0,i);
}

void writenumber(int number)
{
	for(int c=0;c<8;c++)
	{
		clearmatrix();		//vycisteni led matice
		for(int d=0;d<8;d++)
		ledmatrix7219d88_setledon(0,8*c+d);
		_delay_ms(50);
	}
	clearmatrix();
	switch (number)
	{
		case 1:
			writedot(27);
			break;
		case 2:
			writedot(13);
			writedot(41);
			break;
		case 3:
			writedot(27);
			writedot(13);
			writedot(41);
			break;
		case 4:
			writedot(13);
			writedot(41);
			writedot(9);
			writedot(45);
			break;	
		case 5:
			writedot(13);
			writedot(41);
			writedot(9);
			writedot(41);
			writedot(45);
			writedot(27);
			break;
		case 6:
			writedot(1);
			writedot(5);
			writedot(25);
			writedot(29);
			writedot(49);
			writedot(53);
			break;		
	}
	_delay_ms(2000);
}

void writedot(int corner)
{
	ledmatrix7219d88_setledon(0,corner);
	ledmatrix7219d88_setledon(0,corner+1);
	ledmatrix7219d88_setledon(0,corner+8);
	ledmatrix7219d88_setledon(0,corner+9);
}

void lowbattery(void)
{
	clearmatrix();
	for(int i=0;i<5;i++)
	{
		ledmatrix7219d88_setledon(0,10);
		ledmatrix7219d88_setledon(0,11);
		ledmatrix7219d88_setledon(0,12);
		ledmatrix7219d88_setledon(0,13);
		ledmatrix7219d88_setledon(0,18);
		ledmatrix7219d88_setledon(0,21);
		ledmatrix7219d88_setledon(0,26);
		ledmatrix7219d88_setledon(0,29);
		ledmatrix7219d88_setledon(0,34);
		ledmatrix7219d88_setledon(0,37);
		ledmatrix7219d88_setledon(0,42);
		ledmatrix7219d88_setledon(0,43);
		ledmatrix7219d88_setledon(0,44);
		ledmatrix7219d88_setledon(0,45);
		ledmatrix7219d88_setledon(0,51);
		ledmatrix7219d88_setledon(0,52);
		_delay_ms(400);
		clearmatrix();
		_delay_ms(400);
	}
}

void strelka(int led)
{	
	clearmatrix();
	writedot(27);
	switch(led)
	{	case 0:
			ledmatrix7219d88_setledon(0,0);
			ledmatrix7219d88_setledon(0,9);
			ledmatrix7219d88_setledon(0,18);
			break;
		case 1:
			ledmatrix7219d88_setledon(0,8);
			ledmatrix7219d88_setledon(0,17);
			ledmatrix7219d88_setledon(0,18);
			break;
		case 2:
			ledmatrix7219d88_setledon(0,16);
			ledmatrix7219d88_setledon(0,17);
			ledmatrix7219d88_setledon(0,26);
			break;
		case 3:
			ledmatrix7219d88_setledon(0,24);
			ledmatrix7219d88_setledon(0,25);
			ledmatrix7219d88_setledon(0,26);
			break;
		case 4:
			ledmatrix7219d88_setledon(0,32);
			ledmatrix7219d88_setledon(0,33);
			ledmatrix7219d88_setledon(0,34);
			break;
		case 5:
			ledmatrix7219d88_setledon(0,40);
			ledmatrix7219d88_setledon(0,41);
			ledmatrix7219d88_setledon(0,34);
			break;
		case 6:
			ledmatrix7219d88_setledon(0,48);
			ledmatrix7219d88_setledon(0,49);
			ledmatrix7219d88_setledon(0,42);
			break;
		case 7:
			ledmatrix7219d88_setledon(0,56);
			ledmatrix7219d88_setledon(0,49);
			ledmatrix7219d88_setledon(0,42);
			break;
		case 8:
			ledmatrix7219d88_setledon(0,57);
			ledmatrix7219d88_setledon(0,49);
			ledmatrix7219d88_setledon(0,42);
			break;
		case 9:
			ledmatrix7219d88_setledon(0,58);
			ledmatrix7219d88_setledon(0,50);
			ledmatrix7219d88_setledon(0,43);
			break;
		case 10:
			ledmatrix7219d88_setledon(0,59);
			ledmatrix7219d88_setledon(0,51);
			ledmatrix7219d88_setledon(0,43);
			break;
		case 11:
			ledmatrix7219d88_setledon(0,60);
			ledmatrix7219d88_setledon(0,52);
			ledmatrix7219d88_setledon(0,44);
			break;
		case 12:
			ledmatrix7219d88_setledon(0,61);
			ledmatrix7219d88_setledon(0,53);
			ledmatrix7219d88_setledon(0,44);
			break;
		case 13:
			ledmatrix7219d88_setledon(0,62);
			ledmatrix7219d88_setledon(0,54);
			ledmatrix7219d88_setledon(0,45);
			break;
		case 14:
			ledmatrix7219d88_setledon(0,63);
			ledmatrix7219d88_setledon(0,54);
			ledmatrix7219d88_setledon(0,45);
			break;
		case 15:
			ledmatrix7219d88_setledon(0,55);
			ledmatrix7219d88_setledon(0,54);
			ledmatrix7219d88_setledon(0,45);
			break;
		case 16:
			ledmatrix7219d88_setledon(0,47);
			ledmatrix7219d88_setledon(0,46);
			ledmatrix7219d88_setledon(0,37);
			break;
		case 17:
			ledmatrix7219d88_setledon(0,39);
			ledmatrix7219d88_setledon(0,38);
			ledmatrix7219d88_setledon(0,37);
			break;
		case 18:
			ledmatrix7219d88_setledon(0,31);
			ledmatrix7219d88_setledon(0,30);
			ledmatrix7219d88_setledon(0,29);
			break;
		case 19:
			ledmatrix7219d88_setledon(0,23);
			ledmatrix7219d88_setledon(0,22);
			ledmatrix7219d88_setledon(0,29);
			break;
		case 20:
			ledmatrix7219d88_setledon(0,15);
			ledmatrix7219d88_setledon(0,14);
			ledmatrix7219d88_setledon(0,21);
			break;
		case 21:
			ledmatrix7219d88_setledon(0,7);
			ledmatrix7219d88_setledon(0,14);
			ledmatrix7219d88_setledon(0,21);
			break;
		case 22:
			ledmatrix7219d88_setledon(0,6);
			ledmatrix7219d88_setledon(0,14);
			ledmatrix7219d88_setledon(0,21);
			break;
		case 23:
			ledmatrix7219d88_setledon(0,5);
			ledmatrix7219d88_setledon(0,13);
			ledmatrix7219d88_setledon(0,20);
			break;
		case 24:
			ledmatrix7219d88_setledon(0,4);
			ledmatrix7219d88_setledon(0,12);
			ledmatrix7219d88_setledon(0,20);
			break;
		case 25:
			ledmatrix7219d88_setledon(0,3);
			ledmatrix7219d88_setledon(0,11);
			ledmatrix7219d88_setledon(0,19);
			break;
		case 26:
			ledmatrix7219d88_setledon(0,2);
			ledmatrix7219d88_setledon(0,10);
			ledmatrix7219d88_setledon(0,19);
			break;
		case 27:
			ledmatrix7219d88_setledon(0,1);
			ledmatrix7219d88_setledon(0,9);
			ledmatrix7219d88_setledon(0,18);
			break;
	}
}

void uspi_led(void)
{
	clearmatrix();
	max7219_shutdown(0,0);
}

void uspi_mems(void)
{
	mpu6050_setSleepEnabled();
}

void uspi_cpu(void)
{
	set_sleep_mode(SLEEP_MODE_IDLE);
	sleep_mode();
}

void probud_led(void)
{
	max7219_shutdown(0,1);
	_delay_ms(10);
}

void probud_mems(void)
{
	mpu6050_setSleepDisabled();
	_delay_ms(10);
}

void ind_uspani(void)
{
	char led[10]={18,19,20,21,28,35,42,43,44,45};
	for(int i=0;i<5;i++)
	{
		for(int j=0;j<10;j++)
			ledmatrix7219d88_setledon(0,led[j]);
		_delay_ms(100);
		clearmatrix();
		_delay_ms(100);	
	}
}

